/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.grammar.token.definition;

import java.io.Serializable;
import java.util.Vector;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.helpers.IntegerList;
import net.sourceforge.chaperon.parser.RegexAutomate;

/**
 * This class describes a regex character class
 *
 * @author Stephan Michels
 * @version $Revision: 1.2 $ $Date: 2002/05/06 16:22:56 $
 */
public class CharacterClass extends DefinitionElement implements Serializable, Cloneable
{

  private Vector  _childs = new Vector();

  private boolean _negation = false;

  public CharacterClass()
  {
  }

  /**
   * Add a character class element
   *
   * @param element Element, which should be added
   */
  public void addCharacterClassElement(CharacterClassElement element)
  {
    if (element != null)
      _childs.addElement(element);
  }

  /**
   * Returns a character class element
   *
   * @param index Index of the character class element
   *
   * @return Character class element
   */
  public CharacterClassElement getCharacterClassElement(int index)
  {
    return (CharacterClassElement) _childs.elementAt(index);
  }

  /**
   * Returns the count of character class elements
   *
   * @return Count of character class elements
   */
  public int getCharacterClassElementCount()
  {
    return _childs.size();
  }

  /**
   * If the comparing character must match to the elements,
   * or should not match to the elements.<br/>
   * [...]<br/>
   * [^...]
   *
   * @param negation
   */
  public void setNegation(boolean negation)
  {
    _negation = negation;
  }

  public boolean isNegation()
  {
    return _negation;
  }

  /**
   * Convert the character class to a string
   *
   * @return Content of the alternation
   */
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append("[");
    for (int i = 0; i < getCharacterClassElementCount(); i++)
      buffer.append(getCharacterClassElement(i).toString());
    buffer.append("]");
    return buffer.toString();
  }

  public Object clone() throws CloneNotSupportedException
  { 
    CharacterClass clone = new CharacterClass();
    
    clone.setMinOccurs(getMinOccurs());
    clone.setMaxOccurs(getMaxOccurs());

    clone.setNegation(isNegation());
    
    for (int i = 0; i < getCharacterClassElementCount(); i++)
      clone.addCharacterClassElement(getCharacterClassElement(i));

    return clone;
  }
}
